/*
 * progressform.c
 *
 * Manage the backup progress form
 *
 * Copyright (C) 2004 Daniel Thompson <d.thompson@gmx.net> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */
 
#include "rfbackup.h"

static void FrmSetLabel(FormType *form, UInt16 labelID, const Char *string)
{
	UInt16 labelObjectIndex = FrmGetObjectIndex(form, labelID);
	
	FrmHideObject(form, labelObjectIndex);
	FrmCopyLabel(form, labelID, string);
	FrmShowObject(form, labelObjectIndex);
}

static void FrmDrawProgressBar(FormType *form, UInt16 gadgetID, UInt16 offset, UInt16 max)
{
        RectangleType bounds,drawBounds,eraseBounds;
        Coord newWidth;
        UInt16 gadgetObjectIndex;

	if (0 == max) {
		max = 1;
		offset = 0;
	}

	gadgetObjectIndex = FrmGetObjectIndex(form, gadgetID);
        FrmGetObjectBounds(form, gadgetObjectIndex, &bounds);

	drawBounds = bounds;
        newWidth = (long)offset * bounds.extent.x / (long)max;
        if (newWidth < 0) {
        	newWidth = 0;
        } else if (newWidth > bounds.extent.x) {
        	newWidth = bounds.extent.x;
        }
        drawBounds.extent.x = newWidth;
        WinDrawRectangle(&drawBounds,0);

	eraseBounds = bounds;
        eraseBounds.topLeft.x = bounds.topLeft.x + newWidth;
        newWidth = bounds.extent.x - newWidth;
        if (newWidth < 0)
        {
            newWidth = 0;
        }
        eraseBounds.extent.x = newWidth;
        WinEraseRectangle(&eraseBounds,0);
}

static void ProgressFormUpdateDialog(const Char *subTitle, const Char *dbName, UInt16 x, UInt16 max)
{
	FormType *form = FrmGetActiveForm();
	
	if (NULL != subTitle) {
		FrmSetLabel(form, kLabelSubTitle, subTitle);
	}
	if (NULL != dbName) {
		FrmSetLabel(form, kLabelDatabase, dbName);
	}
	FrmDrawProgressBar(form, kGadgetProgressBar, x, max);
}

void ProgressFormDoDialog(UInt16 mode)
{
	Err err = dmErrCorruptDatabase;
	FormPtr form;
	UInt16 volRefNum;
	UInt32 volIterator = vfsIteratorStart;
	Boolean skipReset = false;

	/* assert our preconditions */
	ErrNonFatalDisplayIf(mode != progressFormBackupToVFS && mode != progressFormRestoreFromVFS,
	                     "Invalid backup mode");

	/* bring up the progress form */
	FrmPopupForm(kFormProgress);
	if (!AppHandleEventsUntil(frmOpenEvent)) {
		/* application is shutting down */
		return;
	}
	
	/* update the form */
	form = FrmGetActiveForm();
	/* TODO: set title appropraitely */

	err = VFSVolumeEnumerate(&volRefNum, &volIterator);
	if (errNone == err) {
		err = (mode == progressFormBackupToVFS    ? BakBackupToVFS : BakRestoreFromVFS) (
				volRefNum, "/PALM/RFBackup", ProgressFormUpdateDialog);
	}
	
	switch (err) {
	case expErrEnumerationEmpty:
		(void) FrmAlert(kAlertNoMemoryCard);
		skipReset = true;
		break;
		
	case errNone:
		break;
	
	default:
		ErrAlert(err);
	}

	if (progressFormRestoreFromVFS == mode && !skipReset) {
		(void) FrmAlert(kAlertReset);
		SysReset();
	}

	FrmReturnToForm(0);
}

Boolean ProgressFormHandleEvent(EventPtr event)
{
	Boolean eventHandled = false;
	FormPtr form;
	
	form = FrmGetActiveForm();
	
	switch (event->eType) {
	case frmOpenEvent:
		/* ensure the hashes are never seen by the user */
		FrmSetLabel(form, kLabelSubTitle, "");
		FrmSetLabel(form, kLabelDatabase, "");
		/*FALLTHRU*/
	case frmUpdateEvent:
		FrmDrawForm(form);
		eventHandled = true;
		break;

	default:
		break;
	}

	return eventHandled;
}
 